/*
//         AtTiny85
//          +-\/-+
//    PB5  1|    |8   VCC
//    PB3  2|    |7   PB2  INT0 [SWITCH]
//    PB4  3|    |6   PB1  INT1  PWM1 OCR0B [LED]
//    GND  4|    |5   PB0        PWM0
//          +----+
*/

//Additional modes:
#define DOUBLE_CLICK_MAX_LIGHT
#define TRIPLE_CLICK_STROBE_LIGHT 


#include <avr/io.h>
#include <avr/sleep.h>
#include <util/delay.h>

#if (defined DOUBLE_CLICK_MAX_LIGHT || defined TRIPLE_CLICK_STROBE_LIGHT)
#define ENABLE_ADDITIONAL_MODES
#endif

#define LED PINB1
#define LED_REG OCR0B

#define SW PINB2 
#define SW_DOWN 0

//Less than this interpreted as fast click
#define FAST_CLICK_MS 500
//Delay between click repeat while SW pressed
#define NEXT_CLICK_MS 40

#define MIN_BRIGHT 255
#define MAX_BRIGHT 0

#define BRIGHT_INC255 1
#define BRIGHT_INC200 5
#define BRIGHT_INC100 10

void enablePWM()
{
    /*
    Control Register A for Timer/Counter-0 (Timer/Counter-0 is configured using two registers: A and B)
    TCCR0A is 8 bits: [COM0A1:COM0A0:COM0B1:COM0B0:unused:unused:WGM01:WGM00]
    2<<COM0A0: sets bits COM0A0 and COM0A1, which (in Fast PWM mode) clears OC0A on compare-match, and sets OC0A at BOTTOM
    2<<COM0B0: sets bits COM0B0 and COM0B1, which (in Fast PWM mode) clears OC0B on compare-match, and sets OC0B at BOTTOM
    3<<WGM00: sets bits WGM00 and WGM01, which (when combined with WGM02 from TCCR0B below) enables Fast PWM mode
    */
    TCCR0A = 2<<COM0A0 | 2<<COM0B0 | 3<<WGM00;
   
    /*
    Control Register B for Timer/Counter-0 (Timer/Counter-0 is configured using two registers: A and B)
    TCCR0B is 8 bits: [FOC0A:FOC0B:unused:unused:WGM02:CS02:CS01:CS00]
    0<<WGM02: bit WGM02 remains clear, which (when combined with WGM00 and WGM01 from TCCR0A above) enables Fast PWM mode
    1<<CS00: sets bits CS01 (leaving CS01 and CS02 clear), which tells Timer/Counter-0 to not use a prescalar
    */
    TCCR0B = 0<<WGM02 | 1<<CS00;
   
    /*
    Control Register for Timer/Counter-1 (Timer/Counter-1 is configured with just one register: this one)
    TCCR1 is 8 bits: [CTC1:PWM1A:COM1A1:COM1A0:CS13:CS12:CS11:CS10]
    0<<PWM1A: bit PWM1A remains clear, which prevents Timer/Counter-1 from using pin OC1A (which is shared with OC0B)
    0<<COM1A0: bits COM1A0 and COM1A1 remain clear, which also prevents Timer/Counter-1 from using pin OC1A (see PWM1A above)
    1<<CS10: sets bit CS11 which tells Timer/Counter-1  to not use a prescalar
    */
    TCCR1 = 0<<PWM1A | 0<<COM1A0 | 1<<CS10;
   
    /*
    General Control Register for Timer/Counter-1 (this is for Timer/Counter-1 and is a poorly named register)
    GTCCR is 8 bits: [TSM:PWM1B:COM1B1:COM1B0:FOC1B:FOC1A:PSR1:PSR0]
    1<<PWM1B: sets bit PWM1B which enables the use of OC1B (since we disabled using OC1A in TCCR1)
    2<<COM1B0: sets bit COM1B1 and leaves COM1B0 clear, which (when in PWM mode) clears OC1B on compare-match, and sets at BOTTOM
    */
    GTCCR = 1<<PWM1B | 2<<COM1B0;
}

#ifdef ENABLE_ADDITIONAL_MODES
volatile uint8_t clickscount;
ISR(PCINT0_vect)
{
  //debounce
  _delay_ms(20);
  GIFR |= _BV(PCIF);
  
  if(bit_is_set(PINB, SW))
    clickscount++;
}
#endif

void sleep()
{
  sleep_enable();

  sei();
  sleep_cpu();
  cli();

  sleep_disable();
  
  sei();
}

int main() 
{
  GIMSK |= _BV(PCIE); // Enable Pin Change Interrupts
  PCMSK |= _BV(PCINT2); // Use PB2 as interrupt pin
  ADCSRA &= ~_BV(ADEN); // ADC off
  
  MCUCR = ~_BV(SM0) & _BV(SM1); // Select "Power-down" sleep mode
  DDRB = ~_BV(SW) & _BV(LED); //input for button with pullup
  PORTB = _BV(SW) & ~_BV(LED);//output for led, turn it off
  enablePWM();
  
  uint8_t bright = MIN_BRIGHT;
  uint8_t lastkey = 0;
  uint8_t keydown = 0;

  while(1){
#ifdef ENABLE_ADDITIONAL_MODES    
    clickscount = 0;
#endif
    keydown = bit_is_set(PINB, SW);
    
    if(keydown != lastkey){ //Switch key change state
      if((bright < MIN_BRIGHT) && (keydown == SW_DOWN)){ 
        _delay_ms(FAST_CLICK_MS);
        if(bit_is_set(PINB, SW) != SW_DOWN){
          bright = MIN_BRIGHT;
          LED_REG = bright;
          continue;
        }
      }
      lastkey = keydown;
    }
    
    if(keydown == SW_DOWN){

      if(bright > MAX_BRIGHT){ //Increase light until max while SW pressed
        if(bright <= 100){
          if(bright > BRIGHT_INC100){
            bright -= BRIGHT_INC100;
          }else{
            bright = 0;
          }
        }else
        if(bright <= 200){
           bright -= BRIGHT_INC200;
        }else
           bright -= BRIGHT_INC255;
      }
      
      LED_REG = bright;
      
      if(bright == MIN_BRIGHT - BRIGHT_INC255){
        _delay_ms(FAST_CLICK_MS); //5.5mA

#ifdef ENABLE_ADDITIONAL_MODES
        if(clickscount >= 2) //double fast click
        {

          if(clickscount >= 3){ //triple fast click
#ifdef TRIPLE_CLICK_STROBE_LIGHT          
            clickscount = 0;
            do{ //strobe
              LED_REG = 0;
              _delay_ms(50);
              LED_REG = 255;
              _delay_ms(50);
            }while(!clickscount);
            bright = MIN_BRIGHT;
#endif            
          }else{
#ifdef DOUBLE_CLICK_MAX_LIGHT           
            bright = MAX_BRIGHT;
            LED_REG = bright;
#endif             
          }
        }
#endif        
      }
    }
  
    _delay_ms(NEXT_CLICK_MS); //1.64mA
    
    //If light is off, goto sleep.
    if(bright==MIN_BRIGHT){
      sleep(); //0.3uA
    }
  }
  return 0;
}

